#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fitsio.h>

void printerror(int status);
char* get_next_card(fitsfile* fptr, int hdrnumber);
int dump_file_contents(const char* filename, int);
int duplicate_file_without_data(const char*,const char*);
void Usage(void);

static char* progname;

int main(int argc, char** argv)
{
    char* cmd;

    progname = argv[0];
    if (argc < 2) {
        Usage();
    }

    cmd = argv[1];
    if (strcmp(cmd, "dump") == 0) {
        if (!(argc == 3 || argc == 4)) {
            fprintf(stderr, "Usage: %s dump filename hdrnum?\n", progname);
            exit(1);
        }
        if (argc == 3) {
            dump_file_contents(argv[2], -1);
        } else {
            dump_file_contents(argv[2], strtol(argv[3], NULL, 10));
        }
    } else if (strcmp(cmd, "copy-create") == 0) {
        if (argc != 4) {
            fprintf(stderr, "Usage: %s copy-create src-filename dest-filename\n", progname);
            exit(1);
        }
        duplicate_file_without_data(argv[2], argv[3]);
    } else {
        Usage();
    }

    exit(0);
}

void Usage(void)
{
    fprintf(stderr, "Usage: %s dump filename hdrnum?\n", progname);
    fprintf(stderr, "Usage: %s copy-create src-filename dest-filename\n", progname);
    exit(1);
}

int dump_file_contents(const char* filename, int hdrnumber) 
{
    fitsfile *fptr;
    int status = 0;

    if (fits_open_file(&fptr, filename, READONLY, &status)) {
        printerror(status);
    }

    if (hdrnumber <= 0) {
        int more_headers;
        for (hdrnumber=1, more_headers=1; more_headers; hdrnumber++) {
            char *card;
            card = get_next_card(fptr, hdrnumber);
            if (card == NULL) {
                more_headers = 0;
            } else {
                printf("----------------------------------------\nHeader for HDU %d\n", hdrnumber);
                printf("%s\n", card);
                while ((card = get_next_card(fptr, -1)) != NULL) {
                    printf("%s\n", card);
                }
            }
        }
    } else {
        char *card;
        card = get_next_card(fptr, hdrnumber);
        if (card == NULL) {
            fprintf(stderr, "No header number %d\n", hdrnumber);
            return 0;
        } else {
            printf("%s\n", card);
            while ((card = get_next_card(fptr, -1)) != NULL) {
                printf("%s\n", card);
            }
        }
    }

    if (fits_close_file(fptr, &status))
        printerror(status);

    return status;
}

int duplicate_file_without_data(const char* source, const char* dest)
{
    fitsfile* fptr;
    int status = 0;
    char* buffer;
    int buflen;
    
    buflen = strlen(source)+strlen(dest)+3;
    buffer = (char*)malloc(buflen);
    snprintf(buffer, buflen, "%s(%s)", dest, source);
    fits_create_file(&fptr, buffer, &status);

    if (fits_close_file(fptr, &status))
        printerror(status);
    
    return status;
}

char* get_next_card(fitsfile* fptr, int hdrnumber) 
{
    static int index = -1;
    static int nkeys;
    static char card[FLEN_CARD];
    int status, keypos;

    if (hdrnumber >= 0) {
        // initialise
        if (fits_movabs_hdu(fptr, hdrnumber, NULL, &status)) {
            fprintf(stderr, "Bad header number: %d\n", hdrnumber);
            return NULL;
        }
        if (fits_get_hdrpos(fptr, &nkeys, &keypos, &status)) {
            printerror(status);
        }
        index = 1;
    } else if (index < 0) {
        // error
        fprintf(stderr, "get_next_card: Trying to retrieve next card, but not initialised\n");
        return NULL;
    } else {
        // normal case
        index++;
    }
    if (index > nkeys) {
        return NULL;
    } else {
        if (fits_read_record(fptr, index, card, &status)) {
            printerror(status);
        }
        return card;
    }
}

void printerror(int status)
{
    if (status) {
        fits_report_error(stderr, status);
        exit(status);
    }
    return;
}
